#include "MorsePWM.h"

/*  Copyright (c) 2020 Jaroslaw Weglinski based on Lewis library 
    https://git.defproc.co.uk/DefProc/Lewis
    Changes licensed under the MIT license.*/


void MorsePWM::begin(uint8_t rx_pin, uint8_t tx_pin, uint8_t words_per_minute, uint8_t use_interrupts, uint8_t audio_pin, unsigned int audio_freq) {
  _audio_freq = audio_freq;
  _audio_pin = audio_pin;
  Lewis::begin(rx_pin, tx_pin, words_per_minute, use_interrupts);
}

void MorsePWM::begin(uint8_t rx_pin, uint8_t tx_pin, uint8_t words_per_minute, uint8_t use_interrupts) {
  Lewis::begin(rx_pin, tx_pin, words_per_minute, use_interrupts);
}

size_t MorsePWM::write(uint8_t c) {
  return Lewis::write(c);
}

uint8_t MorsePWM::setWPM(uint8_t words_per_minute) {
  if (words_per_minute >= 1 && words_per_minute <= 60) {
    _pulse_duration = 1200 / words_per_minute;
    return 0;
  } else
    return 1;
}

uint8_t MorsePWM::getWPM(void) {
  return 1200 / _pulse_duration;
}

//modified by including second output for buzzer. Lewis:dot needs to be declared virtual.
void MorsePWM::dot() {
  if ((_audio_pin == 255) || (_use_interrupts == true)) {
    Lewis::dot();
  } else {
    tone(_audio_pin, _audio_freq);
    digitalWrite(_tx_pin, HIGH);
    delay(_pulse_duration * DOT);
    digitalWrite(_tx_pin, LOW);
    noTone(_audio_pin);
    delay(_pulse_duration);
  }
}

//modified by including second output for buzzer. Lewis:dash needs to be declared virtual.
void MorsePWM::dash() {
  if ((_audio_pin == 255) || (_use_interrupts == true)) {
    Lewis::dash();
  } else {
    digitalWrite(_tx_pin, HIGH);
    tone(_audio_pin, _audio_freq);
    delay(_pulse_duration * DASH);
    digitalWrite(_tx_pin, LOW);
    noTone(_audio_pin);
    delay(_pulse_duration);
  }
}

//modified by including second output for buzzer, silencing buzzer after buffer flush and removing listening for incoming data
void MorsePWM::timerISR() 
{
  if (_samePin) {
    // if using one pin:
    if (!_transmitting && _tx_buffer_head != _tx_buffer_tail) {
      // block recieving when we have something in the transmit buffer
      _transmitting = true;
      pinMode(_rx_pin, OUTPUT);
    } else if (_transmitting && _tx_buffer_head == _tx_buffer_tail) {
      // go back to listening when the tx buffer is empty
      _transmitting = false;
      pinMode(_rx_pin, INPUT);
    }
  }

  volatile unsigned long time_now = millis();

  if (_tx_buffer_tail != _tx_buffer_head && (time_now >= _next_tx)) {//} || time_now - _next_tx >= time_now)) {
    // there is data to send out (the tail hasn't caught up with the head)
    // AND the time of next transmit change has passed
    if (_tx_state == LOW) {
      if (_tx_buffer[_tx_buffer_tail] == DOT || _tx_buffer[_tx_buffer_tail] == DASH) {
        // change to high if DOT or DASH
        // and set the _next_tx depending on length
        _tx_state = HIGH;
        digitalWrite(_tx_pin, _tx_state);
        tone(_audio_pin, _audio_freq);
        _next_tx = (time_now + (_pulse_duration * _tx_buffer[_tx_buffer_tail]));
        // but don't move the buffer position, we need the trailing rest first
      } else {
        // it's a rest, keep low and wait for the next character
        _tx_state = LOW;
        digitalWrite(_tx_pin, _tx_state);
        noTone(_audio_pin);
        _next_tx = (time_now + (_pulse_duration * _tx_buffer[_tx_buffer_tail]));
        // and increment the buffer tail position
        _tx_buffer_tail = (_tx_buffer_tail + 1) % MORSE_TX_BUFFER_SIZE;
      }
    } else {
      // it's a high state, and we need to make the trailing rest
      _tx_state = LOW;
      digitalWrite(_tx_pin, _tx_state);
      noTone(_audio_pin);
      _next_tx = (time_now + _pulse_duration);
      // and increment the buffer tail position
      _tx_buffer_tail = (_tx_buffer_tail + 1) % MORSE_TX_BUFFER_SIZE;
    }
  } else if (_tx_buffer_tail == _tx_buffer_head) {
    //settle to off after buffer flush
    digitalWrite(_tx_pin, LOW);
    noTone(_audio_pin);
  }
}
